/* ----------------------------------------------------------------------
   miniMD is a simple, parallel molecular dynamics (MD) code.   miniMD is
   an MD microapplication in the Mantevo project at Sandia National 
   Laboratories ( http://software.sandia.gov/mantevo/ ). The primary 
   authors of miniMD are Steve Plimpton and Paul Crozier 
   (pscrozi@sandia.gov).

   Copyright (2008) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This library is free software; you 
   can redistribute it and/or modify it under the terms of the GNU Lesser 
   General Public License as published by the Free Software Foundation; 
   either version 3 of the License, or (at your option) any later 
   version.
  
   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
   Lesser General Public License for more details.
    
   You should have received a copy of the GNU Lesser General Public 
   License along with this software; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.  See also: http://www.gnu.org/licenses/lgpl.txt .

   For questions, contact Paul S. Crozier (pscrozi@sandia.gov). 

   Please read the accompanying README and LICENSE files.
---------------------------------------------------------------------- */

#include "stdio.h"
#include "math.h"
#include "force.h"

Force::Force() {}
Force::~Force() {}

void Force::setup()
{
  cutforcesq = cutforce*cutforce;
}

void Force::compute(Atom &atom, Neighbor &neighbor, int me)
{
  int i,j,k,nlocal,nall,numneigh;
  float xtmp,ytmp,ztmp,delx,dely,delz,rsq;
  float sr2,sr6,force;
  int *neighs;
  float **x,**f;

  nlocal = atom.nlocal;
  nall = atom.nlocal + atom.nghost;
  x = atom.x;
  f = atom.f;

  // clear force on own and ghost atoms

  for (i = 0; i < nall; i++) {
    f[i][0] = 0.0f;
    f[i][1] = 0.0f;
    f[i][2] = 0.0f;
  }

  // loop over all neighbors of my atoms
  // store force on both atoms i and j
  
  for (i = 0; i < nlocal; i++) {
    neighs = neighbor.firstneigh[i];
    numneigh = neighbor.numneigh[i];
    xtmp = x[i][0];
    ytmp = x[i][1];
    ztmp = x[i][2];
    for (k = 0; k < numneigh; k++) {
      j = neighs[k];
      delx = xtmp - x[j][0];
      dely = ytmp - x[j][1];
      delz = ztmp - x[j][2];
      rsq = delx*delx + dely*dely + delz*delz;
      if (rsq < cutforcesq) {
	sr2 = 1.0f/rsq;
	sr6 = sr2*sr2*sr2;
	force = sr6*(sr6-0.5f)*sr2;
	f[i][0] += delx*force;
	f[i][1] += dely*force;
	f[i][2] += delz*force;
	f[j][0] -= delx*force;
	f[j][1] -= dely*force;
	f[j][2] -= delz*force;
      }
    }
  }
}
